#!/usr/bin/env python3
import ctypes
from ctypes import *
import syslog
from enum import Enum

ppro_add = 9
ppro_del = 10
fun_ppro = 5

class kysec_ppro_type(Enum):
    PPRO_SUBS = 1
    PPRO_KEY = 2

class kysec_func_status(Enum):
	FUNC_DISABLED = 0
	FUNC_ENABLED = 1

class kysec_ppro_kernel(Structure):
    _fields_ = [
        ('path', ctypes.c_char * 4096),
        ('hash', ctypes.c_char * 65),
        ('type', ctypes.c_int)
    ]

class KeyProcessControl:
    def __init__(self):
        self.extend_so = CDLL("libkysec_extend.so.0")

    def kysec_process_enable(self):
        ret = self.extend_so.kysec_getstatus()
        if ret != 2:
            syslog.syslog(syslog.LOG_DEBUG, "kysec get status error")
            return -1

        ret = self.extend_so.kysec_get_func_status(c_int(fun_ppro))
        if ret == 1 or ret == 2:
            return 0

        if ret == 0:
            ret = self.extend_so.kysec_set_func_status(c_int(fun_ppro), c_int(kysec_func_status.FUNC_ENABLED.value))
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec set func status error")
                return -1

            ret = self.extend_so.kysec_conf_set("kysec_ppro", c_int(kysec_func_status.FUNC_ENABLED.value))
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec set conf error")
                return -1

        return 0

    def kysec_ppro_add(self, path):
        ret = self.extend_so.kysec_ppro_add(c_char_p(bytes(path, encoding = "utf8")), c_int(kysec_ppro_type.PPRO_KEY.value))
        if ret == -1:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro add error")
            return -1

        self.extend_so.kysec_ppro_read_node_with_path.restype = POINTER(kysec_ppro_kernel)
        try:
            p = self.extend_so.kysec_ppro_read_node_with_path(c_char_p(bytes(path, encoding = "utf8")))
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro read node error")
            return -1

        if p:
            syslog.syslog(syslog.LOG_DEBUG, "add type = {}, path = {}, hash = {}".format(p.contents.type, str(p.contents.path, "utf8"), str(p.contents.hash, "utf8")))
            ret = self.extend_so.kysec_ppro_communicate_with_kernel(c_int(ppro_add), p.contents.hash, p.contents.type)
        else:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro communicate error")
            return -1

        return 0

    def kysec_ppro_delete(self, path):
        self.extend_so.kysec_ppro_read_node_with_path.restype = POINTER(kysec_ppro_kernel)
        try:
            p = self.extend_so.kysec_ppro_read_node_with_path(c_char_p(bytes(path, encoding = "utf8")))
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro read node error")
            return -1

        if p:
            syslog.syslog(syslog.LOG_DEBUG, "del type = {}, path = {}".format(p.contents.type, str(p.contents.path, "utf8")))
            ret = self.extend_so.kysec_ppro_communicate_with_kernel(c_int(ppro_del), p.contents.hash, p.contents.type)
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec ppro communicate error")
                return -1

        ret = self.extend_so.kysec_ppro_remove(c_char_p(bytes(path, encoding = "utf8")))
        if ret != 0:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro remove error")
            return -1

        return 0

    def kysec_ppro_get(self):
        self.extend_so.kysec_ppro_read_all_data.restype = POINTER(kysec_ppro_kernel)
        count = c_int(0)
        list = self.extend_so.kysec_ppro_read_all_data(byref(count))
        return list, count.value

    def kysec_ppro_clear(self):
        list, count = self.kysec_ppro_get()
        for i in range(0, count):
            # print(list[i].type, list[i].path, list[i].hash)
            if list[i].type == kysec_ppro_type.PPRO_KEY.value:
                ret = self.kysec_ppro_delete(str(list[i].path, "utf8"))
                if ret != 0:
                    syslog.syslog(syslog.LOG_DEBUG, "kysec ppro delete error")
                    return -1
        return 0
